#include <stdlib.h>
#include <SDL/SDL.h>

Uint32 GetPixel(SDL_Surface *screen, int x, int y )
{
    Uint32 color;

    if ( SDL_MUSTLOCK(screen) ) {
        if ( SDL_LockSurface(screen) < 0 ) {
            return 0;
        }
    }
    
    Uint32 *bufp;

    bufp = (Uint32 *)screen->pixels + y*screen->pitch/4 + x;
    color = *bufp;

    if ( SDL_MUSTLOCK(screen) ) {
        SDL_UnlockSurface(screen);
    }
    return color;
}

int main(int argc, char *argv[])
{
  if( argc==1 )
  {
    exit( 1 );
  }
  
  char filename[255];
  
  


  SDL_Surface *bmp, *bmp32bit;
  SDL_PixelFormat pixelformat;
  
  SDL_Init( SDL_INIT_VIDEO );
  atexit( SDL_Quit );
  
  //Load BMP...
  //bmp = SDL_LoadBMP( argv[1] );

  

  sprintf( filename, "%s.bmp", argv[1] );
  bmp = SDL_LoadBMP( filename );
  
  //convert to 32bit
  
		pixelformat.palette=NULL;
		pixelformat.BitsPerPixel=32;
		pixelformat.BytesPerPixel=4;
//#if SDL_BYTEORDER == SDL_LIL_ENDIAN
		pixelformat.Rshift=0;
		pixelformat.Gshift=8;
		pixelformat.Bshift=16;

  /*
#else
		pixelformat.Rshift=24;
		pixelformat.Gshift=16;
		pixelformat.Bshift=8;
#endif*/
		pixelformat.Ashift=0;
		pixelformat.Rmask=0xff<<pixelformat.Rshift;
		pixelformat.Gmask=0xff<<pixelformat.Gshift;
		pixelformat.Bmask=0xff<<pixelformat.Bshift;
		pixelformat.Amask=0;
		pixelformat.Rloss=0;
		pixelformat.Gloss=0;
		pixelformat.Bloss=0;
		pixelformat.Aloss=8;
		pixelformat.colorkey=0;
		pixelformat.alpha=0; 
    
  bmp32bit = SDL_ConvertSurface( bmp, &pixelformat, SDL_SWSURFACE );
  

  Uint16 width, height;
  Uint32 curColor, lastPixel;
  bool   setPixel;
  Uint16 pixelCount;
  FILE   *outputFile;
  

  Uint16 *outBuffer;
  Uint16 *rlepointer, *datapointer;
  
  outBuffer = new Uint16[0xFFFF];  //lets say the max outbuffer size is 2*64K, keen can only read 64k and u never reach 1:2 with huffman
  
  datapointer = outBuffer;
  
  rlepointer = outBuffer;
  rlepointer += (bmp32bit->h)+2;
  
  *datapointer++ = bmp32bit->h;
  *datapointer++ = bmp32bit->w;
  
  
  
  for( height = 0; height < bmp32bit->h; height++ )
  {
    pixelCount = 0;
    setPixel   = false;
    lastPixel  = GetPixel( bmp32bit, 0, height );  //set start-pixel
    
    //set line-start-offset :)
    *datapointer++ = (rlepointer - outBuffer)*2;     //get start-offset
    
    for( width = 0; width < bmp32bit->w; width++ )
    {
      curColor = GetPixel( bmp32bit, width, height );
      if( lastPixel == curColor )
      {
         pixelCount++;
      } else {
         //write to outmem
         *rlepointer++ = pixelCount;
         printf( "%X", pixelCount );
         
         setPixel  ^= true;  //switch between true/false
         pixelCount = 1;
         lastPixel  = curColor;
      }
    }
    
    printf( "\n" );
    
    //output last PixelCount?!? (do we really need this)
    *rlepointer++ = pixelCount;
    //output 0xFFFF (=end of line)
    *rlepointer++ = 0xFFFF;
  }
  
  
  //write buffer to disc
  sprintf( filename, "%s.bin", argv[1] );
  outputFile = fopen( filename, "wb" );
  fwrite( outBuffer, (rlepointer-outBuffer)*2, 1, outputFile );
  fclose( outputFile );
  
  
  
  
  delete [] outBuffer;

 
  exit( 0 );
}
